﻿-------------------------------------------------------------------------------
-- A few "local global" variables for this lua-file:                         --
-------------------------------------------------------------------------------
local TIIT_name        = "Toxyna's 'I interrupted that'";
local TIIT_short_name  = "TIIT";
local TIIT_version     = "0.0.18"
local TIIT_date        = "2012/10/20"
local TIIT_color       = "|cffafdfd0";   -- addon-color for the mesages
local L                = select(2, ...); -- For internationalization
local TIIT_optionPanel;  -- interface addons options frame
local Options;           -- short cut for ToxynasIInterruptedThat_Options


-------------------------------------------------------------------------------
-- Provide default values:                                                   --
-------------------------------------------------------------------------------
local TIIT_Defaults = {
    ---------------------------------------------------------------------------
    -- Default options for the persitant saved variables:                    --
    ---------------------------------------------------------------------------
    ["Options"] = {
        ["MasterOnOff"] = "on",        -- addon is active
        ["version"]     = TIIT_version,
        ["debug"]       = false,
        ["mode"]        = "Emote",     -- addon's output channel is emote
        ["message"]     = L["default-message"],
                                       -- message for other channels than emote
        ["announceAll"] = false,       -- announce interrupts from all members
        ["abilities"]   = {},          -- all gathered interrupt abilities
    },
    ---------------------------------------------------------------------------
    -- later on here could follow other default options ...                  --
    ---------------------------------------------------------------------------
};

-------------------------------------------------------------------------------
-- Prints the message in TIIT's color:                                       --
-------------------------------------------------------------------------------
local function c_print(message)
    DEFAULT_CHAT_FRAME:AddMessage(TIIT_color .. message .. "|r");
end

-------------------------------------------------------------------------------
-- Prints the two parts, the first in TIIT_color, the second normal:         --
-------------------------------------------------------------------------------
local function cc_print(part1, part2)
    DEFAULT_CHAT_FRAME:AddMessage(TIIT_color .. part1 .. "|r" .. part2);
end

-------------------------------------------------------------------------------
-- Prints the message:                                                       --
-------------------------------------------------------------------------------
local function n_print(message)
    DEFAULT_CHAT_FRAME:AddMessage(message);
end

-------------------------------------------------------------------------------
-- Prints a table:                                                           --
-------------------------------------------------------------------------------
local function print_table(my_table)
    local k, v;
    for k, v in pairs(my_table) do
        c_print(k .. " -> " .. tostring(v));
    end
end

-------------------------------------------------------------------------------
-- Table copy:                                                               --
-------------------------------------------------------------------------------
local function table_copy(t)
    local t2 = {};
    for k,v in pairs(t) do
        t2[k] = v;
    end
    return t2;
end


--===========================================================================--


-------------------------------------------------------------------------------
-- This function is called, when the addon is loading:                       --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_OnLoad(self)
    ---------------------------------------------------------------------------
    -- Greeting on load or reload:                                           --
    ---------------------------------------------------------------------------
    c_print(TIIT_name .. " -- " .. L["greeting"]);

    ---------------------------------------------------------------------------
    -- Install slash command /tiit:                                          --
    ---------------------------------------------------------------------------
    SlashCmdList["TOXYNAS_I_INTERRUPTED_THAT"] =
            ToxynasIInterruptedThat_SlashCommand;
    SLASH_TOXYNAS_I_INTERRUPTED_THAT1 = "/tiit";
    self:RegisterEvent("VARIABLES_LOADED");

    ---------------------------------------------------------------------------
    -- Enable an interface options addons panel:                             --
    ---------------------------------------------------------------------------
    ToxynasIInterruptedThat_InitOptionsPanel(self);
end

-------------------------------------------------------------------------------
-- Enable an interface options addons panel:                                 --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_InitOptionsPanel(self)
    TIIT_optionPanel = CreateFrame("Frame", "ToxynasIInterruptedThatConfig",
            InterfaceOptionsFramePanelContainer);

    ---------------------------------------------------------------------------
    -- Set the name for the Category for the Panel:                          --
    ---------------------------------------------------------------------------
    --TIIT_optionPanel.name = TIIT_short_name .. " " .. TIIT_version;
    TIIT_optionPanel.name = "Toxyna's IIT";

    ---------------------------------------------------------------------------
    -- When the player clicks okay, run this function:                       --
    ---------------------------------------------------------------------------
    TIIT_optionPanel.okay =
        function (self)
            ToxynasIInterruptedThat_OptionPanel_Close();
        end;

    ---------------------------------------------------------------------------
    -- When the player clicks cancel, run this function:                     --
    ---------------------------------------------------------------------------
    TIIT_optionPanel.cancel =
        function (self)
            ToxynasIInterruptedThat_Option_CancelOrLoad();
        end;

    ---------------------------------------------------------------------------
    -- When the player clicks default, run this function:                    --
    ---------------------------------------------------------------------------
    TIIT_optionPanel.default =
        function (self)
            ToxynasIInterruptedThat_Option_Default();
        end;

    TIIT_optionPanel_fill();

    ---------------------------------------------------------------------------
    -- Add the TIIT_optionPanel to the Interface Options:                    --
    ---------------------------------------------------------------------------
    InterfaceOptions_AddCategory(TIIT_optionPanel);
end

-------------------------------------------------------------------------------
-- Local widgets from the interface addons options:                          --
-------------------------------------------------------------------------------
local btnMasterOnOff;
local modeDropDown;
local btnAnnounceAllOnOff;
local message_edit_box;
local default_button;
local btnDebugOnOff;

-------------------------------------------------------------------------------
-- Fill the interface addons options panel:                                  --
-------------------------------------------------------------------------------
function TIIT_optionPanel_fill()
    TIIT_optionPanel:Hide();

    ---------------------------------------------------------------------------
    -- page title:
    ---------------------------------------------------------------------------
    local title = TIIT_optionPanel:CreateFontString(nil, "ARTWORK",
            "GameFontNormalLarge");
    title:SetPoint("TOPLEFT", 16, -16);
    title:SetText(TIIT_name .. " " .. L["interface-addons-options-version"] ..
            " " .. TIIT_version);

    ---------------------------------------------------------------------------
    -- MasterOnOff:
    ---------------------------------------------------------------------------
    btnMasterOnOff = CreateFrame("CheckButton",
            "TIITConfigMasterOnOffButton", TIIT_optionPanel,
            "OptionsBaseCheckButtonTemplate");
    btnMasterOnOff:SetPoint("TOPLEFT", 16, -35);
    btnMasterOnOff:SetScript("OnClick", function(TIIT_optionPanel)
        if btnMasterOnOff:GetChecked() then
            PlaySound("igMainMenuOptionCheckBoxOn");
        else
            PlaySound("igMainMenuOptionCheckBoxOff");
        end
    end);
    local btnMasterOnOffText = btnMasterOnOff:CreateFontString(nil, "ARTWORK",
            "GameFontHighlight");
    btnMasterOnOffText:SetPoint("LEFT", btnMasterOnOff, "RIGHT", 0, 1);
    btnMasterOnOffText:SetText(L["interface-addons-options-addon-active"]);

    ---------------------------------------------------------------------------
    -- Dropdown für den mode
    ---------------------------------------------------------------------------
    local modeText = TIIT_optionPanel:CreateFontString(nil, "OVERLAY",
            "GameFontNormal");
    modeText:SetPoint("TOPLEFT", btnMasterOnOff, "BOTTOMLEFT", 0, 1);
    modeText:SetJustifyH("LEFT");
    modeText:SetHeight(18);
    modeText:SetWidth(120);
    modeText:SetText(L["interface-addons-options-output-mode"] .. ":");

    modeDropDown = CreateFrame("Frame", "ToxynasIInterruptedThatOutputMode",
            TIIT_optionPanel, "UIDropDownMenuTemplate");
    modeDropDown:SetPoint("TOPLEFT", modeText, "TOPRIGHT", 0, 3) -- 16, 3
    local function itemOnClick(self)
        local selected = self.value;
        UIDropDownMenu_SetSelectedValue(modeDropDown, selected);
    end
    local function dropDown_Initialize()
        local i, mode;
        for i, mode in pairs({"Emote", "Self", "Auto", "ForceParty", "Say"}) do
            local info = UIDropDownMenu_CreateInfo();
            info.text = mode;
            info.value = mode;
            info.func = itemOnClick;
            UIDropDownMenu_AddButton(info);
        end
    end
    UIDropDownMenu_Initialize(modeDropDown, dropDown_Initialize);
    UIDropDownMenu_SetWidth(modeDropDown, 160);
    UIDropDownMenu_JustifyText(modeDropDown, "LEFT");

    ---------------------------------------------------------------------------
    -- AnnounceAllOnOff:
    ---------------------------------------------------------------------------
    btnAnnounceAllOnOff = CreateFrame("CheckButton",
            "TIITConfigAnnounceAllOnOffButton", TIIT_optionPanel,
            "OptionsBaseCheckButtonTemplate");
    btnAnnounceAllOnOff:SetPoint("TOPLEFT", modeText, "BOTTOMLEFT", 0, 1);
    btnAnnounceAllOnOff:SetScript("OnClick", function(TIIT_optionPanel)
        if btnAnnounceAllOnOff:GetChecked() then
            PlaySound("igMainMenuOptionCheckBoxOn");
        else
            PlaySound("igMainMenuOptionCheckBoxOff");
        end
    end);
    local btnAnnounceAllOnOffText = btnAnnounceAllOnOff:CreateFontString(nil,
            "ARTWORK", "GameFontHighlight");
    btnAnnounceAllOnOffText:SetPoint("LEFT", btnAnnounceAllOnOff, "RIGHT",
            0, 1);
    btnAnnounceAllOnOffText:SetText(
            L["interface-addons-options-announce-foreign-interrupts"]);

    ---------------------------------------------------------------------------
    -- EditBox for the message:                                              --
    ---------------------------------------------------------------------------
    local message_edit_boxText = TIIT_optionPanel:CreateFontString(nil,
            "OVERLAY", "GameFontNormal");
    message_edit_boxText:SetPoint("TOPLEFT", btnAnnounceAllOnOff, "BOTTOMLEFT",
            0, 1);
    message_edit_boxText:SetJustifyH("LEFT");
    message_edit_boxText:SetHeight(18);
    message_edit_boxText:SetWidth(120);
    message_edit_boxText:SetText(L["interface-addons-options-message"] .. ":");

    message_edit_box = CreateFrame("EditBox",
            "TIITConfigMessageEditBox", TIIT_optionPanel,
            "InputBoxTemplate");
    message_edit_box:SetPoint("TOPLEFT", message_edit_boxText, "TOPRIGHT",
                16, 3)
    message_edit_box:SetSize(400, 20);
    message_edit_box:SetAutoFocus(false);

    ---------------------------------------------------------------------------
    -- Default-Button:                                                       --
    ---------------------------------------------------------------------------
    default_button = CreateFrame("Button", "DefaultButton", TIIT_optionPanel,
            "UIPanelButtonTemplate");
    default_button:SetSize(80, 20);
    default_button:SetPoint("BOTTOMLEFT", 10, 10);
    default_button.text = _G["DefaultButtonText"];
    default_button.text:SetText("Default");
    default_button:SetScript("OnClick", function(self, button, down)
        ToxynasIInterruptedThat_Option_Default()
    end);

    ---------------------------------------------------------------------------
    -- Debug-CheckButton:                                                    --
    ---------------------------------------------------------------------------
    btnDebugOnOff = CreateFrame("CheckButton",
            "TIITConfigDebugOnOffButton", TIIT_optionPanel,
            "OptionsBaseCheckButtonTemplate");
    btnDebugOnOff:SetPoint("BOTTOMLEFT", default_button, "BOTTOMLEFT", 0, 20);
    btnDebugOnOff:SetScript("OnClick", function(TIIT_optionPanel)
        if btnDebugOnOff:GetChecked() then
            PlaySound("igMainMenuOptionCheckBoxOn");
        else
            PlaySound("igMainMenuOptionCheckBoxOff");
        end
    end);
    local btnAnnounceAllOnOffText = btnDebugOnOff:CreateFontString(nil,
            "ARTWORK", "GameFontHighlight");
    btnAnnounceAllOnOffText:SetPoint("LEFT", btnDebugOnOff, "RIGHT", 0, 1);
    btnAnnounceAllOnOffText:SetText(
            L["interface-addons-options-debug-on-off"]);

end

-------------------------------------------------------------------------------
-- function TIIT_set_message() validates the user input and sets the new     --
-- message, if it is validated.                                              --
-------------------------------------------------------------------------------
local function TIIT_set_message(message)
    if message == Options["message"] then
        return;
    end

    local has_mob    = nil ~= string.find(message, "MOB",    1, true);
    local has_spell  = nil ~= string.find(message, "SPELL",  1, true);
    local has_person = nil ~= string.find(message, "PERSON", 1, true);
    if has_mob and has_spell and has_person then
        --c_print("'" .. message .. "' " .. L["tiit-setmsg-valid"]);
        Options["message"] = message;
        cc_print(L["tiit-setmsg-set-to"] .. ": '", message .. "'");
    else
        c_print("'" .. message .. "' " .. L["tiit-setmsg-not-valid"]);
        cc_print(L["tiit-setmsg-stays"] .. ": '", Options["message"] .. "'");
        StaticPopup_Show("TOXYNAS_I_INTERRUPTED_THAT_FALSE_MESSAGE", message);
    end

    ---------------------------------------------------------------------------
    -- Set the changes in the interface addons options:                      --
    ---------------------------------------------------------------------------
    ToxynasIInterruptedThat_Option_CancelOrLoad();
end

-------------------------------------------------------------------------------
-- When the player clicks okay on the interface options addons panel:        --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_OptionPanel_Close()
    xpcall(function()
        -- MasterOnOff:
        if btnMasterOnOff:GetChecked() then
            Options["MasterOnOff"] = "on";
        else
            Options["MasterOnOff"] = "off";
        end

        -- mode:
        Options["mode"] = UIDropDownMenu_GetSelectedValue(modeDropDown);

        -- announceAll:
        Options["announceAll"] = 1 == btnAnnounceAllOnOff:GetChecked();

        -- message:
        TIIT_set_message(message_edit_box:GetText());

        -- debug:
        Options["debug"] = 1 == btnDebugOnOff:GetChecked();
    end, geterrorhandler())
end

-------------------------------------------------------------------------------
-- When the player clicks cancel on the interface options addons panel:      --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_Option_CancelOrLoad()
    xpcall(function()
        -- MasterOnOff:
        btnMasterOnOff:SetChecked(Options["MasterOnOff"] == "on");

        -- mode:
        UIDropDownMenu_SetSelectedValue(modeDropDown, Options["mode"]);
        UIDropDownMenu_SetText(modeDropDown, Options["mode"]);

        -- announceAll:
        btnAnnounceAllOnOff:SetChecked(Options["announceAll"]);

        -- message:
        message_edit_box:SetText(Options["message"]);
        message_edit_box:ClearFocus();

        -- debug:
        btnDebugOnOff:SetChecked(Options["debug"]);
    end, geterrorhandler())
end

-------------------------------------------------------------------------------
-- When the player clicks default on the interface options addons panel:     --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_Option_Default()
    xpcall(function()
        Options = table_copy(TIIT_Defaults["Options"]);
        ToxynasIInterruptedThat_Option_CancelOrLoad();
    end, geterrorhandler())
end


--===========================================================================--


-------------------------------------------------------------------------------
-- Init dialogs:                                                             --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_InitDialogs()
    StaticPopupDialogs["TOXYNAS_I_INTERRUPTED_THAT_FALSE_MESSAGE"] = {
        text = "'%s' " .. L["tiit-setmsg-not-valid"] .. "\n\n" ..
                L["tiit-setmsg-stays"] .. ": '" .. Options["message"] .. "'",
        button1 = "Ok",
        timeout = 0,
        whileDead = true,
        hideOnEscape = true,
        preferredIndex = 3,
    }
end

-------------------------------------------------------------------------------
-- Persitance Handling                                                       --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_Loaded(...)
    ---------------------------------------------------------------------------
    -- If there are no variables saved, use the default values:              --
    ---------------------------------------------------------------------------
    if (not ToxynasIInterruptedThat_Options) then
        ToxynasIInterruptedThat_Options = table_copy(TIIT_Defaults["Options"]);
        c_print(TIIT_name .. " " .. L["persistance-no-options-found"]);

    ---------------------------------------------------------------------------
    -- If there are variables saved, but they have no version:               --
    ---------------------------------------------------------------------------
    elseif (ToxynasIInterruptedThat_Options["version"] == nil) then
        ToxynasIInterruptedThat_Options = table_copy(TIIT_Defaults["Options"]);
        c_print(TIIT_name .. " " ..
                L["persistance-options-without-version"] .. " " ..
                L["persistance-generate-new-options"]);

    ---------------------------------------------------------------------------
    -- If the saved variables have an earlier addon version, copy the known  --
    -- values and get the rest from the default:                             --
    ---------------------------------------------------------------------------
    elseif (ToxynasIInterruptedThat_Options["version"] ~= TIIT_version) then
        c_print(TIIT_name .. " " .. L["persistance-elder-options-found"]);
        local options_temp = table_copy(TIIT_Defaults["Options"]);
        local k, v;
        for k, v in pairs(ToxynasIInterruptedThat_Options) do
            if (nil ~= TIIT_Defaults["Options"][k]) then
                options_temp[k] = v;
            end
        end
        options_temp["version"] = TIIT_version;
        ToxynasIInterruptedThat_Options = table_copy(options_temp);
    end

    ---------------------------------------------------------------------------
    -- Save the local variable Options as a short cut for                    --
    -- ToxynasIInterruptedThat_Options:                                      --
    ---------------------------------------------------------------------------
    Options = ToxynasIInterruptedThat_Options;

    ---------------------------------------------------------------------------
    -- Set the changes in the interface addons options:                      --
    ---------------------------------------------------------------------------
    ToxynasIInterruptedThat_Option_CancelOrLoad();

    ---------------------------------------------------------------------------
    -- Init dialogs:                                                         --
    ---------------------------------------------------------------------------
    ToxynasIInterruptedThat_InitDialogs();
end


--===========================================================================--


-------------------------------------------------------------------------------
-- Reacts at unfiltered combat log events.                                   --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_CombatLogEventUnfiltered(...)
    ---------------------------------------------------------------------------
    -- if the addon is not active : return                                   --
    ---------------------------------------------------------------------------
    if Options["MasterOnOff"] == "off" then
        return;
    end

    ---------------------------------------------------------------------------
    -- get some of the event parameters:                                     --
    ---------------------------------------------------------------------------
    local eventType            = select(2, ...);
    local interruptingUserGuiD = select(4, ...);
    local interruptingUser     = select(5, ...);
    local interruptedUser      = select(9, ...);
    local spellID              = select(15, ...);
    local interruptedSpell     = GetSpellLink(spellID);
    local interruptSpellId     = select(12, ...);
    local interruptSpell       = GetSpellLink(interruptSpellId);

    ---------------------------------------------------------------------------
    -- handel the event SPELL_INTERRUPT:                                     --
    ---------------------------------------------------------------------------
    if eventType == "SPELL_INTERRUPT" then
        -----------------------------------------------------------------------
        -- Save the interrupting spell in the database, if it is not         --
        -- known yet:                                                        --
        -- (To avoid some errors, only store interrupts by the player and    --
        -- it's members.)                                                    --
        -----------------------------------------------------------------------
        if interruptingUser == UnitName("player") or
           UnitInRaid(interruptingUser) or
           UnitInParty(interruptingUser) then
            local interruptingUserClass  =
                    select(2, GetPlayerInfoByGUID(interruptingUserGuiD));
            if nil ~= interruptingUserClass then
                if nil == Options["abilities"][interruptingUserClass] then
                    Options["abilities"][interruptingUserClass] = {};
                end

                if nil == Options["abilities"][interruptingUserClass][interruptSpellId] then
                    Options["abilities"][interruptingUserClass][interruptSpellId] =
                            interruptSpell;
                    if Options["debug"] then
                        c_print(TIIT_name .. " new spell added to database: " ..
                                interruptingUserClass .. " " ..
                                interruptSpellId .. " " .. interruptSpell);
                    end
                end
            else
                if Options["debug"] then
                    c_print("interruptingUserClass nil:");
                    c_print("eventType             = " .. tostring(eventType));
                    c_print("interruptingUserGuiD  = " .. tostring(interruptingUserGuiD));
                    c_print("interruptingUser      = " .. tostring(interruptingUser));
                    c_print("interruptedUser       = " .. tostring(interruptedUser));
                    c_print("spellID               = " .. tostring(spellID));
                    c_print("interruptedSpell      = " .. tostring(interruptedSpell));
                    c_print("interruptSpellId      = " .. tostring(interruptSpellId));
                    c_print("interruptSpell        = " .. tostring(interruptSpell));
                    c_print("interruptingUserClass = " .. tostring(interruptingUserClass));
                end
                return; -- Ignore interrupts by unknown objects (pets...)
            end
        end

        -----------------------------------------------------------------------
        -- if someone else interrupts and we don't want to tell: return      --
        -----------------------------------------------------------------------
        if not Options["announceAll"] and
            interruptingUser ~= UnitName("player") then
                return;
        end

        -----------------------------------------------------------------------
        -- if the interrupter is not in our party/raid: return               --
        -----------------------------------------------------------------------
        if UnitInRaid("player")  and not UnitInRaid(interruptingUser)  or
           UnitInParty("player") and not UnitInParty(interruptingUser) then
            return;
        end

        -----------------------------------------------------------------------
        -- handle the 'Emote' mode:
        -----------------------------------------------------------------------
        if Options["mode"] == 'Emote' then
            -------------------------------------------------------------------
            -- the following placeholder can be used in the Emote-Message:   --
            -- #INTERRUPTED_USER#  the user or mob, that was casting and was --
            --                     interrupted by the player                 --
            -- #INTERRUPTED_SPELL# the spell that user or mob was casting    --
            -- #INTERRUPTING_USER# the player's name                         --
            -------------------------------------------------------------------
            local message; -- the message that will be shown as a emote
            local default_message = L["emote-default-message"];

            -------------------------------------------------------------------
            -- the player has interrupted:
            -------------------------------------------------------------------
            if interruptingUser == UnitName("player") then
                local localizedClass, englishClass = UnitClass("player");
                if "SHAMAN" == englishClass then
                    if interruptSpellId == 57994 then -- Wind Shear
                        message = L["emote-shaman-wind-shear"];
                    else
                        message = default_message;
                    end
                elseif "ROGUE" == englishClass then
                    -- Rogues can also interrupt via Gauge, Solar Plexus,
                    -- Cheap Shot, or Garotte, but only Kick is a
                    -- "SPELL_INTERRUPT" and I yet have no idea, how to get
                    -- the other four abilities.
                    if interruptSpellId == 1766 then -- Kick
                        message = L["emote-rogue-kick"];
                    else
                        message = default_message;
                    end
                elseif "MAGE" == englishClass then
                    if interruptSpellId == 29443 then -- Counterspell
                        message = L["emote-mage-counterspell"];
                    else
                        message = default_message;
                    end
                elseif "WARRIOR" == englishClass then
                    if interruptSpellId == 6552 then -- Pummel
                        message = L["emote-warrior-pummel"];
                    elseif interruptSpellId == 102060 then -- Disrupting Shout
                        message = L["emote-warrior-disrupting-shout"];
                    else
                        message = default_message;
                    end
                elseif "PALADIN" == englishClass then
                    if interruptSpellId == 31935 then -- Avenger's Shield
                        message = L["emote-paladin-avengers-shield"];
                    elseif interruptSpellId == 96231 then -- Rebuke
                        message = L["emote-paladin-rebuke"];
                    else
                        message = default_message;
                    end
                elseif "DRUID" == englishClass then
                    if interruptSpellId == 78675 then -- Solar Beam
                        message = L["emote-druid-solar-beam"];
                    else
                        message = default_message;
                    end
                elseif "DEATHKNIGHT" == englishClass then
                    if interruptSpellId == 47528 then -- Mind Freeze
                        message = L["emote-deathknight-mind-freeze"];
                    elseif interruptSpellId == 32747 then -- Interrupt
                        message = default_message;
                        -- if someone can provide me a more specific description
                        -- of the deathknight's action while using this skill, I
                        -- will add a better emote.
                    else
                        message = default_message;
                    end
                elseif "MONK" == englishClass then
                    -- The ability Spear Hand Strike was found with id 116705
                    -- and 116709 in game.
                    if interruptSpellId == 116705 then -- Spear Hand Strike
                        message = L["emote-monk-spear-hand-strike"];
                    elseif interruptSpellId == 116709 then -- Spear Hand Strike
                        message = L["emote-monk-spear-hand-strike"];
                    else
                        message = default_message;
                    end
                else
                    message = default_message;
                end
                --[[
                www.wowhead.com/spell=47528 engslisch
                de.wowhead.com/spell=47528  deutsch
                ]]

                --local genderCode = UnitSex(interruptingUser);
                local genderCode = UnitSex("player");
                local she_or_he_t = { L["emote-it"], L["emote-he"], L["emote-she"] };
                local she_or_he = she_or_he_t[genderCode];
                local her_or_his_t = { L["emote-its"], L["emote-his"], L["emote-her"] };
                local her_or_his = her_or_his_t[genderCode];
                message = string.gsub(message, '#SHEorHE#',  she_or_he);
                message = string.gsub(message, '#HERorHIS#', her_or_his);

            -------------------------------------------------------------------
            -- someone else has interrupted:
            -------------------------------------------------------------------
            else
                message = L["emote-admire-foreign-interrupts"];
            end

            -------------------------------------------------------------------
            -- Debug informations about unhandled interrupting spells:       --
            -------------------------------------------------------------------
            if Options["debug"] and message == default_message then
                c_print(TIIT_name .. " unknown interrupting spell found:" ..
                        interruptingUserClass .. " interrupted with " ..
                        interruptSpellId .. " " .. interruptSpell .. ".");
            end

            -------------------------------------------------------------------
            -- replace the placeholders:
            -------------------------------------------------------------------
            message = string.gsub(message, '#INTERRUPTED_USER#',
                    interruptedUser);
            message = string.gsub(message, '#INTERRUPTED_SPELL#',
                    interruptedSpell);
            message = string.gsub(message, '#INTERRUPTING_USER#',
                    interruptingUser);


            -------------------------------------------------------------------
            -- Send the mesage:                                              --
            -- In case that the player is on a battle ground and is          --
            -- stealthed, we do not generate the emote! Instead a simple     --
            -- message just for the player himself is generated.             --
            -------------------------------------------------------------------
            if IsStealthed() and "pvp" == select(2, IsInInstance()) then
                n_print(message);
            else
                SendChatMessage(message, "EMOTE");
            end

        -----------------------------------------------------------------------
        -- other channel than 'Emote':
        -----------------------------------------------------------------------
        else
            local numberOfGroupMembers = GetNumGroupMembers();

            -------------------------------------------------------------------
            -- Determine the channel for the mode 'Self', 'ForceParty',      --
            -- 'Auto', and 'Say':                                            --
            -------------------------------------------------------------------
            local onlySelf = false; -- true: only text for the player
            local autoChannel;      -- the channel to use if not onlySelf
            local instanceType = select(2, IsInInstance());
                                    -- "pvp" if on a battle ground

            -------------------------------------------------------------------
            -- Determine the channel for the mode 'Self':                    --
            -------------------------------------------------------------------
            if Options["mode"] == 'Self' then
                onlySelf = true;
            -------------------------------------------------------------------
            -- Determine the channel for the mode 'ForceParty':              --
            -------------------------------------------------------------------
            elseif Options["mode"] == 'ForceParty' then
                if numberOfGroupMembers > 0 then
                    autoChannel = "PARTY";
                else
                    onlySelf = true;
                end
            -------------------------------------------------------------------
            -- Determine the channel for the mode 'Auto':                    --
            -------------------------------------------------------------------
            elseif Options["mode"] == 'Auto' then
                if numberOfGroupMembers == 0 then
                    onlySelf = true;
                elseif numberOfGroupMembers <= 5 then
                    autoChannel = "PARTY";
                else
                    if instanceType == "pvp" then
                        autoChannel = "BATTLEGROUND";
                    else
                        autoChannel = "RAID"
                    end
                end
            -------------------------------------------------------------------
            -- Determine the channel for the mode 'Say':                     --
            -------------------------------------------------------------------
            elseif Options["mode"] == 'Say' then
                -- on battle grounds the mode 'Say' is disabled:
                if instanceType == "pvp" then
                    onlySelf = true;
                else
                    autoChannel = "SAY";
                end
            -------------------------------------------------------------------
            -- Only for safety, if anything above got wrong, only a message  --
            -- for the player is generated:                                  --
            -------------------------------------------------------------------
            else
                if Options("debug") then
                    c_print(TIIT_name .. " error: mode '" .. Options["mode"] ..
                            "' not handled in function " ..
                            "ToxynasIInterruptedThat_CombatLogEventUnfiltered"
                            .. "!");
                end
                onlySelf = true;
            end

            -------------------------------------------------------------------
            -- determine output string:                                      --
            -------------------------------------------------------------------
            local message = Options["message"];
            message = string.gsub(message, 'MOB',    interruptedUser      );
            message = string.gsub(message, 'SPELL',  GetSpellLink(spellID));
            message = string.gsub(message, 'PERSON', interruptingUser     );

            -------------------------------------------------------------------
            -- Send the message:                                             --            -------------------------------------------------------------------
            if onlySelf then
                n_print(message);
            else
                SendChatMessage(message, autoChannel);
            end
        end
    ---------------------------------------------------------------------------
    -- handel the event SPELL_AURA_APPLIED:                                  --
    ---------------------------------------------------------------------------
    --[[
    elseif eventType == "SPELL_AURA_APPLIED" and Options["debug"] and
           interruptingUser == UnitName("player") then
        if spellID == 31935 then
            c_print("Aura 31935 applied - Avenger's Shield on " .. interruptedUser);
        elseif spellID == 24259 then
            c_print("Aura 24259 applied - Spell Lock on " .. interruptedUser);
        --elseif spellID == 12311 then
        --    c_print("Aura 12311 applied - unknown 1 on " .. interruptedUser);
        --elseif spellID == 12958 then
        --    c_print("Aura 12958 applied - unknown 2 on " .. interruptedUser);
        else
            c_print("unknown Aura applied: " .. tostring(spellID) .. " " .. tostring(interruptedSpell) .. " on " .. tostring(interruptedUser));
        end
    ]]
    end
end



--===========================================================================--


-------------------------------------------------------------------------------
-- function TIIT_help() shows the addon's options to the user.               --
-------------------------------------------------------------------------------
local function TIIT_help()
    c_print(TIIT_name .. ' ' .. L["interface-addons-options-version"] .. " " ..
            TIIT_version .. ' ' .. L["tiit-help-from"] .. " " .. TIIT_date);
    c_print(L["tiit-help-channel-text"]);
    c_print(L["tiit-help-tiit-help"]);
    c_print(L["tiit-help-tiit-info"]);
    c_print(L["tiit-help-tiit-toggle"] .. " " .. TIIT_name .. " " ..
            L["tiit-help-tiit-toggle-on-or-off"]);
    c_print(L["tiit-help-tiit-allmembers"]);
    c_print(L["tiit-help-tiit-msg"]);
    c_print(L["tiit-help-tiit-setmsg"]);
    c_print(L["tiit-help-tiit-party"]);
    c_print(L["tiit-help-tiit-debug"]);
end

-------------------------------------------------------------------------------
-- The function TIIT_announce_output_channel() tells the output channel to   --
-- the user.                                                                 --
-------------------------------------------------------------------------------
local function TIIT_announce_output_channel()
    cc_print(L["tiit-info-output-channel-is"] .. ": ", Options["mode"]);
end

-------------------------------------------------------------------------------
-- The function TIIT_show_info() show's some state of the addon to the user. --
-------------------------------------------------------------------------------
local function TIIT_show_info()
    if Options["MasterOnOff"] == "off" then
        c_print(TIIT_name .. " " .. L["tiit-info-not-active"]);
    else
        c_print(TIIT_name .. " " .. L["tiit-info-informations"] .. ":");
        -----------------------------------------------------------------------
        -- mode:
        -----------------------------------------------------------------------
        TIIT_announce_output_channel();

        -----------------------------------------------------------------------
        -- announceAll:
        -----------------------------------------------------------------------
        local onOrOff;
        if Options["announceAll"] then
            onOrOff = L["tiit-info-announce-all-on"];
        else
            onOrOff = L["tiit-info-announce-all-off"];
        end
        cc_print(L["interface-addons-options-announce-foreign-interrupts"] ..
                ": ", onOrOff);

        -----------------------------------------------------------------------
        -- message:
        -----------------------------------------------------------------------
        cc_print(L["tiit-info-message-announce"] .. ':|r', Options["message"]);
    end
end

-------------------------------------------------------------------------------
-- The function TIIT_toggle_addon_active() toggles between activated and     --
-- inactivated state of the addon.                                           --
-------------------------------------------------------------------------------
local function TIIT_toggle_addon_active()
    if Options["MasterOnOff"] == "on" then
        Options["MasterOnOff"] = "off";
        c_print(TIIT_name .. " " .. L["tiit-toggle-off"]);
    else
        Options["MasterOnOff"] = "on";
        c_print(TIIT_name .. " " .. L["tiit-toggle-on"]);
    end

    ---------------------------------------------------------------------------
    -- Set the changes in the interface addons options:                      --
    ---------------------------------------------------------------------------
    ToxynasIInterruptedThat_Option_CancelOrLoad();
end

-------------------------------------------------------------------------------
-- The function TIIT_toggle_allmembers() toggles between telling about       --
-- interrupts from other players and not telling so.                         --
-------------------------------------------------------------------------------
local function TIIT_toggle_allmembers()
    if Options["announceAll"] then
        Options["announceAll"] = false;
        c_print(TIIT_name .. " - " .. L["tiit-allmembers-off"]);
    else
        Options["announceAll"] = true;
        c_print(TIIT_name .. " - " .. L["tiit-allmembers-on"]);
    end
end

-------------------------------------------------------------------------------
-- Shows the message for modes other than emote to the user:                 --
-------------------------------------------------------------------------------
local function TIIT_show_message()
    cc_print(L["tiit-msg"] .. ": ", Options["message"]);
end

-------------------------------------------------------------------------------
-- Reacts on the command /tiit and handels user input starting with /tiit.   --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_SlashCommand(command)
    local first_command = gsub(command, "%s*([^%s]+).*", "%1");
    local second_command = gsub(command, "%s*([^%s]+)%s*(.*)", "%2");
    first_command = string.lower(first_command);  -- 'eMotE' -> 'emote'

    if first_command == "" or first_command == "help" then
        TIIT_help();
    elseif first_command == "info" then
        TIIT_show_info();
    elseif first_command == "toggle" then
        TIIT_toggle_addon_active();
    elseif first_command == "say" then
        Options["mode"] = 'Say';
        TIIT_announce_output_channel();
    elseif first_command == "self" then
        Options["mode"] = 'Self';
        TIIT_announce_output_channel();
    elseif first_command == "auto" then
        Options["mode"] = 'Auto';
        TIIT_announce_output_channel();
    elseif first_command == "emote" then
        Options["mode"] = 'Emote';
        TIIT_announce_output_channel();
    elseif first_command == "party" or first_command == "forceparty" then
        Options["mode"] = 'ForceParty';
        TIIT_announce_output_channel();
    elseif (first_command == "allmembers") then
        TIIT_toggle_allmembers();
    elseif (first_command == "msg") then
        TIIT_show_message();
    elseif first_command == 'setmsg' then
        TIIT_set_message(second_command);
    elseif first_command == 'debug' then -- hidden debug option
        Options["debug"] = not Options["debug"];
        if Options["debug"] then
            c_print(TIIT_name .. " debug ist set to true");
        else
            c_print(TIIT_name .. " debug ist set to false");
        end
    elseif first_command == 'abilities' or first_command == 'interrupts' then
        c_print(TIIT_name .. " " .. L["tiit-abilities-in-database"]);
        for k, v in pairs(Options["abilities"]) do
            for kk, vv in pairs(v) do
               c_print(k .. ': ' .. kk .. ' ' .. vv);
            end
        end
    else
        n_print(TIIT_color .. TIIT_name .. " " .. L["slash-parameter"] ..
                 " |r" ..first_command .. " " ..
                 TIIT_color .. L["slash-unknown-parameter"] .. ".|r");
    end

    ---------------------------------------------------------------------------
    -- Set the changes in the interface addons options:                      --
    ---------------------------------------------------------------------------
    ToxynasIInterruptedThat_Option_CancelOrLoad();
end

